/* file csputil.h
 *
 * NIST PIV Demo CSP -- Cryptographic Service Provider
 *
 * Designed and Developed by Electrosoft (www.electrosoft-inc.com) based on requirements
 * from the National Institute of Standards and Technology (NIST).
 *
 * This file codes and documents headers for many utility functions.
 *
 */

#ifndef _CSPUTIL_H_
#define _CSPUTIL_H_       /**< csputil.h inclusion tag. */

BOOL simpleSignData(PROV_CTX *pProvCtx, DWORD dwKeySpec, BYTE *pData, DWORD dataLen,
              BYTE *pSignature, DWORD *pSignatureLen, SCARDHANDLE cardHandle, char* cachedPIN);

BOOL decryptData(PROV_CTX *pProvCtx, BYTE *pData, DWORD dataLen,
              BYTE *pSignature, DWORD *pSignatureLen, SCARDHANDLE cardHandle, char* cachedPIN);

BOOL getAlgorithm(ALG_ID algId, ALGORITHM *algorithm);

int piv_login2(PROV_CTX *pProvCtx, char *pindata, int pinlen, SCARDHANDLE cardHandle);

BOOL loadUserKey(PROV_CTX *pProvCtx, KEY_INFO *pKeyInfo, SCARDHANDLE cardHandle);

BOOL openPIVContainer(PROV_CTX *pProvCtx, SCARDHANDLE *cardHandle);

BOOL closePIVContainer(PROV_CTX *pProvCtx, SCARDHANDLE *cardHandle);

BOOL piv_login(PROV_CTX *pProvCtx, char *pp_pin, DWORD keySpec, SCARDHANDLE cardHandle);

// The following is only for debug
int Base64Encode(const BYTE *pbSrcData, int nSrcLen, LPSTR szDest, int *pnDestLen, DWORD dwFlags);

// Oneofexponent functions ...not all are used
BOOL ImportPlainSessionBlobModified(HCRYPTPROV hProv,
                            HCRYPTKEY hPrivateKey,
                            ALG_ID dwAlgId,
                            LPBYTE pbKeyMaterial,
                            DWORD dwKeyMaterial,
                            HCRYPTKEY *hSessionKey);

BOOL CreateRealKey(LPTSTR szProvider,
                                 DWORD dwProvType,
                                 LPTSTR szContainer,
                                 DWORD dwKeySpec,
                                 HCRYPTPROV *hProv,
                                 HCRYPTKEY *hPrivateKey);


/*
BOOL CreatePrivateExponentOneKey(LPTSTR szProvider,
                                 DWORD dwProvType,
                                 LPTSTR szContainer,
                                 DWORD dwKeySpec,
                                 HCRYPTPROV *hProv,
                                 HCRYPTKEY *hPrivateKey);

BOOL GenerateSessionKeyWithAlgorithm(HCRYPTPROV hProv,
                                     ALG_ID Alg,
                                     HCRYPTKEY *hSessionKey);

BOOL DeriveSessionKeyWithAlgorithm(HCRYPTPROV hProv,
                                   ALG_ID Alg,
                                   LPBYTE lpHashingData,
                                   DWORD dwHashingData,
                                   HCRYPTKEY *hSessionKey);

BOOL ExportPlainSessionBlob(HCRYPTKEY hPublicKey,
                            HCRYPTKEY hSessionKey,
                            LPBYTE *pbKeyMaterial,
                            DWORD *dwKeyMaterial);

BOOL ImportPlainSessionBlob(HCRYPTPROV hProv,
                            HCRYPTKEY hPrivateKey,
                            ALG_ID dwAlgId,
                            LPBYTE pbKeyMaterial,
                            DWORD dwKeyMaterial,
                            HCRYPTKEY *hSessionKey);


BOOL CreatePrivateExponentOneKey(LPTSTR szProvider,
                                 DWORD dwProvType,
                                 LPTSTR szContainer,
                                 DWORD dwKeySpec,
                                 HCRYPTPROV *hProv,
                                 HCRYPTKEY *hPrivateKey);

*/

int GZipInflate(unsigned char *pSrc, unsigned long ulSrcSize, unsigned char **ppDest, unsigned long *pulDestSize);

#endif


